/*
 *	ezinstaller.c
 *	Author: Tim Niiler
 * 	Date: 3/12/04
 *	Version: 0.1
 *	License: GPL
 *		NOTE: Companies or individuals that either do not "believe in" the GPL
 *		or do not abide by it MAY NOT use this code for ANY purpose as they will
 *		be violating copyright law. Otherwise, so long as you agree to the GPL
 *		have at it, you may modify this code as you wish with the understanding
 *		that you keep all comments and credits intact.
 *
 *	Purpose:
 *		This code is designed to act as an easy way to install Linux packages
 *		on any distribution.  Ahem...well, at least on Vector Linux and Slackware
 *		to start with.  But I am trying to write the code in such a way as to 
 *		minimize the distro dependencies in the C portion of it.
 *	
 *		As such, the code calls several distro dependent scripts which the users
 *		may find much easier to modify to fit their distro of choice.  Again, at
 *		the time of this writing, these scripts have only been written for 
 *		Vector Linux and Slackware.
 *	
 *	Disclaimers:
 *		As it has been years since I used C (and I never really liked it), at this 
 *		point I can't guarantee the stability of the code.  But darn it, I'm trying
 *		to get it to the point where it's robust.  As such, any help fellow developers
 * 		can give would be most welcome, especially concerning PROPER and CONVENTIONAL
 *		use of pointers and the like.
 *
 *		As per usual, you use this code at your own risk.
 */
#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>
#include <string.h>

gchar *array[500];
gchar *array1[500];
gchar array2[500]=" ";
gchar array3[500]=" ";
gchar ftpsite[500]="ftp://ftp.ibiblio.org/pub/linux/distributions/vectorlinux/veclinux-4.0/packages/";
gchar manifestsite[500]="ftp://ftp.ibiblio.org/pub/linux/distributions/vectorlinux/veclinux-soho-4.0/stable/niiler/installer/";

gchar setDebug[1] = "1";
gchar cleanUp[1] = "0";

int arraySize;
int ftpsize;
int nPackages;
int allPackages;
int okInstall = 1;

/* Add memory here */
struct menuConts
{
        gchar cat[50];
	gchar md5[500];
        gchar xpix[50];
        gchar meth[50];
        gchar text[500];
        gchar where[3500];
} programs[100]; 

struct ftpStuff
{
	gchar where[1000];
} ftparray[100]; 

struct ftpStuff ftpout[100];
struct ftpStuff currentPackages[500];
struct ftpStuff ftpPackages[500];

struct packageThere
{
	int response;
	int which;
} packageOK; 

void initArr() 
{
	/* Initialize two global pointer arrays */
	*array = array2;
	*array1 = array3;
}

void substr( char sIn[], char sOut[], int start) {
        int i;
        int j=0;
        char C;
        for (i=start;i< strlen(sIn); i++) {
                C=sIn[i];
                sOut[j]=C;
                j++;
        }
        sOut[j]='\0';
}


void deblank( char * l )
{
	/* Remove blanks and initial tabs from a line */
	int i;
	char *p;
   
	i = strspn( l, "\t" ); /* find first  nonblank after tab*/
	if ( i > 0 )
	{
		for( p = l; ; p++ )
		{
			*p = *(p+i);
			if (*p == '\0') break;
		}
	}
	i = strspn( l, " " ); /* find first  nonblank after space*/
	if ( i > 0 )
	{
		for( p = l; ; p++ )
		{
			*p = *(p+i);
        		 if (*p == '\0') break;
      		}
   	}
	if ( ( ( i = strlen( l ) ) - 1 ) >= 0 )
	{
		l[i] = ( l[i] == ' ' ) ? '\0' : l[i];
	}
}

void writeFile( char inputFile[], char text[])
{
	/* Write text to a file */
        FILE *inputFilePointer;
        inputFilePointer = fopen(inputFile,"w");
        fprintf(inputFilePointer,"%s",text);
	fclose(inputFilePointer);
}


/* Note: I would really like to be able to condense the next to subroutines into a single
	subroutine, but I can't figure out how to get C to pass an array back from a 
	subroutine.  When I try using pointers, unless I use the results instantly, 
	memory locations seem to be overwritten.  So much for portable code....So anyhow
	any help on this would be MUCH appreciated.
*/

static int doSplit(gchar *searchString, gchar *token)
{
	/* Split a file...in this program, this function is called for separating lines */
  int loop;

  array[0]=strtok(searchString,token);
  if(array[0]==NULL) {
	if (strcmp(setDebug,"11")==0) {
	    printf("No test to search.\n");
	}
	return 0;
  } else {
	  for(loop=1;loop<500;loop++)
	   {
	    array[loop]=strtok(NULL,token);
	    if(array[loop]==NULL)
	      break;
	   }
	return loop;
  }

}

static int doSplitLine(gchar *searchString, gchar *token)
{
	/* Split a line */
  int loop;

  array1[0]=strtok(searchString,token);
  if(array1[0]==NULL) {
	if (strcmp(setDebug,"11")==0) {
	    printf("No test to search.\n");
	}
	return 0;
  } else {
	for(loop=1;loop<500;loop++)
   	{
   	 array1[loop]=strtok(NULL,token);
   	 if(array1[loop]==NULL)
   	   break;
   	}
	return loop;
  }

}


void toggle_button_callback (GtkWidget *widget, int data)
{
	/* Decide what gets included for download based on which button is pressed */
    if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget)))
    {
        /* If control reaches here, the toggle button is down */
	strcpy(ftpout[data].where,ftparray[data].where);

    } else {
	strcpy(ftpout[data].where,"null");
        /* If control reaches here, the toggle button is up */
    }
}



static GtkWidget *xpm_label_box (gchar *xpm_filename,
                                gchar *label_text)
{
	/* Copied from the GTK+ tutorial for making pixmap buttons */

        GtkWidget *box;
        GtkWidget *label;
        GtkWidget *image;

        /* Create box for image and label */
        box = gtk_hbox_new (FALSE, 0);
        gtk_container_set_border_width (GTK_CONTAINER (box), 2);

        /* Now on to the image stuff */
        image = gtk_image_new_from_file (xpm_filename);

        /* Create a label for the icon */
        label = gtk_label_new (label_text);

        /* Pack the image and label into the box */
        gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 3);
        gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 3);

        gtk_widget_show (image);
        gtk_widget_show (label);

        return box;
}

int callback3 (char * newpackage,
			char * oldpackage)
{
	
	static GtkWidget *window3;
	GtkWidget *label;
	char labelTxt1[1000] = "The package you have selected:\n";
	char labelTxt2[500] = "\nis similar to the package currently installed:\n";
	char labelTxt3[200] = "\nDo you want to replace the currently installed package\ninstall this package alongside the other,\nor cancel the installation of this package?";

	strcat(labelTxt2,oldpackage);
	strcat(labelTxt2,labelTxt3);
	strcat(labelTxt1,newpackage);
	strcat(labelTxt1,labelTxt2);

	window3= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window3), TRUE);
	gtk_window_set_title (GTK_WINDOW (window3), "You must choose, Grasshopper");
	gtk_container_set_border_width (GTK_CONTAINER (window3), 0);
	gtk_widget_set_size_request (window3, 400, 150);

	label = gtk_label_new (labelTxt1);
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window3)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window3), "Replace", 1);
	gtk_dialog_add_button (GTK_DIALOG (window3), "Install", 2);
	gtk_dialog_add_button (GTK_DIALOG (window3), "Cancel", 3);

	gint result = gtk_dialog_run (GTK_DIALOG (window3) );
	if (strcmp(setDebug,"11")==0) {
		printf("Result = %i\n",result);
	}

	gtk_widget_destroy (window3);
	return result;

}


void isPackageThereAlready(gchar *newpackage)
{
	gchar *result;
	int okInstall = 2;
	int j;
/* Add memory here */
/* Currently limited to 100 packages */
	gchar packname[100];
	gchar *newprefix;
	gchar *oldpackage;

	strcpy(packname,newpackage);

	newprefix =strtok(packname,"-"); 
	if (strcmp(setDebug,"11")==0) {
		printf("newpackage = %s\n",newpackage);
		printf("newprefix = %s\n",newprefix);
	}
	packageOK.which = -1;
	for (j=0; j< nPackages; j++) {
		oldpackage = currentPackages[j].where;
		result=strstr(oldpackage,newprefix);
 		if (result == NULL) {
			/* Do Nothing */
		} else {
			okInstall = callback3(newpackage,oldpackage);
			packageOK.which = j;
			break;
		}
	}
	packageOK.response = okInstall;
}

void getManifest() {

        gchar inputFileName[50]="package_manifest.txt";
        gchar C;
/* If program segfaults, increase size of text */
/* Add memory here */
        gchar text[40000];
        gchar token1[]="\n";
        gchar token2[]=";";
        FILE *inputFilePointer;
        int inputFileSize;
        int i;
	int res1;
	int res2;

	/* First get the manifest information into a data structure */
	if (strcmp(setDebug,"10")==0) {
		printf("Getting package manifest\n");
	}
        inputFilePointer = fopen(inputFileName,"r");
        inputFileSize = 0;
        while (fscanf(inputFilePointer,"%c",&C) != -1) text[inputFileSize++] = C;
	fclose(inputFilePointer);


        res1=doSplit(text,token1); 

	if (res1 == 0) {
		if (strcmp(setDebug,"10")==0) {
			printf("No data in file");
		}
	}

	arraySize = res1;
	if (strcmp(setDebug,"10")==0) {
		printf("Lines = %i\n",res1);
	}
        for (i=1; i< res1-1; i++) {
		if (strcmp(setDebug,"10")==0) {
			printf("%i: Getting data...",i);
		}
                res2=doSplitLine(array[i],token2);
		if (res2 == 0) {
			if (strcmp(setDebug,"11")==0) {
				printf("No more data");
			}
		}
                strcpy(programs[i].cat,array1[0]);
                strcpy(programs[i].meth,array1[1]);
		strcpy(programs[i].md5,array1[2]);
                strcpy(programs[i].xpix,array1[3]);
                strcpy(programs[i].text,array1[4]);
                strcpy(programs[i].where,array1[5]);
		if (strcmp(setDebug,"10")==0) {
			printf("%s: Line %i OK...\n",programs[i].text,i);
		}
        }

	/* Next see what packages are currently installed and put this into a data structure */
	if (strcmp(setDebug,"10")==0) {
		printf("Loading list of installed packages...\n");
	}
	system("./listpackages.sh");
	strcpy(inputFileName,"installed_packages.txt");
	inputFilePointer = fopen(inputFileName,"r");
	inputFileSize = 0;
        while (fscanf(inputFilePointer,"%c",&C) != -1) text[inputFileSize++] = C;
	fclose(inputFilePointer);

        res1=doSplit(text,token1);
	nPackages = res1-1;
	for (i=0; i< nPackages; i++) {
		strcpy(currentPackages[i].where,array[i]);
/*		printf("Package#%i = %s\n",i,currentPackages[i].where); */
	}

	/* Next load up the packages in the pkglist.txt file */
	if (strcmp(setDebug,"10")==0) {
		printf("Loading general list of packages...\n");
	}
	strcpy(inputFileName,"pkglist.txt");
	inputFilePointer = fopen(inputFileName,"r");
	inputFileSize = 0;
        while (fscanf(inputFilePointer,"%c",&C) != -1) text[inputFileSize++] = C;
	fclose(inputFilePointer);

        res1=doSplit(text,token1);
	allPackages = res1-1;
	for (i=0; i< allPackages; i++) {
		strcpy(ftpPackages[i].where,array[i]);
	}

	/* Next find out what the ftpsite is */
	if (strcmp(setDebug,"10")==0) {
		printf("Loading config.txt...\n");
	}
	strcpy(inputFileName,"config.txt");
	inputFilePointer = fopen(inputFileName,"r");
	inputFileSize = 0;
        while (fscanf(inputFilePointer,"%c",&C) != -1) text[inputFileSize++] = C;
	fclose(inputFilePointer);

        res1=doSplit(text,token1);
	strcpy(ftpsite,array[0]);
	strcpy(manifestsite,array[1]);
	strcpy(setDebug,array[2]);
	strcpy(cleanUp,array[2]);
	if (strcmp(setDebug,"11")==0) {
		printf("setDebug = %s\n",setDebug);
	}

}

static void callback2 (GtkWidget *widget,
			gpointer data)
{
/* Add memory here */
	int i;
	gchar *ptext[10000];	/* Text to be written to file current_packages.txt */
	gchar text[10000]= " "; /* Real array that ptext points to */
	gchar *prmtxt[10000]; 	/* Text to be written to file removed_packages.txt */
	gchar rmtxt[10000];	/* Real array the prmtxt points to */
	char fname[100] = "current_packages.txt";
	char rfname[100] = "removed_packages.txt";
	gchar where[200] = " ";	
	gchar *pwhere[200];
	gchar *ppackage[100];	/* List of packages that are already installed */
	gchar package[100] = " "; /* Real array...etc... */
	gchar *ppackname[100];
	gchar packname[100] = " ";
	gchar packname1[100];
	gchar oldpack[100] = " ";
	gchar *poldpack[100];
	gchar temp[500] = " ";
	int res=0;
	int count=0;
	int rcount=0; 
	int okRemove=0; 
	int pkgCount=0;

	GtkWidget *window2;
	GtkWidget *label;

	/* Initialize pointers */
	*ptext = text;
	*prmtxt = rmtxt;
	*ppackname = packname;
	*ppackage = package;
	*poldpack = oldpack;
	*pwhere = where;

	
	for (i=0;i< ftpsize; i++) {
		if (strcmp(setDebug,"11")==0) {
			printf("ftpout[%i] = %s\n",i,ftpout[i].where);
		}
		res = 0;
		if (strcmp(ftpout[i].where,"null") == 0) {
			/* Do nothing */
		} else {
			/* first determine what the actual package names are */
			strcpy(where,ftpout[i].where);
			if (strcmp(setDebug,"11")==0) {
				printf("where = %s\n",where);
			}

			*pwhere = strrchr(where, '/');
			strcpy(packname,*pwhere);
			substr(packname,packname1,1);
			if (strcmp(setDebug,"11")==0) {
				printf("packname = %s\n",packname1);
			}
			strcpy(package,packname1); 
			*ppackage =strtok(package,"-"); 

			/* compare package names to see if the same or similar package */
			/* BTW, this code could be much smarter and could send a flag to 
			 * callback3 to indicate whether a replacement was recommended or not.
			 * Of course, some folks might not want that...Also, a flag might be 
			 * set to remove safeties and just install everything indicated or
			 * even worse, to remove prior packages and then install overtop.
			 * A last concern is replacing or updating ezinstaller OR GTK.  I suspect
			 * dire consequences in replacing executable code that lives where the
			 * current code is running, but I have not been brave enough to try
			 * it on my box.  Maybe when I have some good backups... 
			 */
			isPackageThereAlready(packname1);
			okInstall = packageOK.response;
			*poldpack = currentPackages[packageOK.which].where;
			
			/* Then get the packages in question */
			if (strcmp(ftpout[i].where,"null")!=0) {
				if (okInstall == 1) {	
					okRemove = 1;
					/* Replace the old file TODO */
					strcpy(temp,"xterm -geometry 70x10 -e wget --passive-ftp --directory-prefix=packages ");
					strcat(temp, ftpout[i].where);
					if (strcmp(setDebug,"11")==0) {
						printf("Replacing package....\n");
					}
					if (count == 0) {
						strcpy(text,packname);
					} else {
						strcat(text,"\n");
						strcat(text,packname);
					}
					count++;
					if (rcount == 0) {
						strcpy(*prmtxt,*poldpack);
					} else {
						strcat(*prmtxt,"\n");
						strcat(*prmtxt,*poldpack);
					}
					rcount++;
					pkgCount++;
					system(temp); 
				} else if (okInstall == 2) { 
					/* Just install the package */
					strcpy(temp,"xterm -geometry 70x10 -e wget --passive-ftp --directory-prefix=packages ");
					strcat(temp, ftpout[i].where);
					if (strcmp(setDebug,"11")==0) {
						printf("Getting package....\n");
					}
					if (count == 0) {
						strcpy(text,packname);
					} else {
						strcat(text,"\n");
						strcat(text,packname);
					}
					count++;
					pkgCount++;
					system(temp); 
				} else {		
					/* Don't get the package or install it */
					if (strcmp(setDebug,"11")==0) {
						printf("Skipping package\n");
					}
				}
			}
			if (strcmp(setDebug,"11")==0) {
				printf("%s\n",text);
			}
		}
	}
	if (pkgCount > 0) {
		if (okRemove == 1) {
			writeFile(rfname,*prmtxt);
		} 
		if (strcmp(setDebug,"11")==0) {
			printf("text = %s\n",text);
		}
		writeFile(fname,text);
		system("./xinstallpackages.sh");
		if (strcmp(cleanUp,"1")==0) {
			system("./cleanup.sh");
		}
		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "Alert!");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 80);
	
		label = gtk_label_new ("\n\nInstallation Finished\n");
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);

		gtk_dialog_add_button (GTK_DIALOG (window2), "OK", 1);
		gtk_dialog_run ( GTK_DIALOG (window2) );
		gtk_widget_destroy (window2);
	}

}

static void callback1 (GtkWidget *widget,
			gpointer data)
{
	static GtkWidget *window2;
	GtkWidget *scrolled_window;
	GtkWidget *table;
	GtkWidget *button;
	GtkWidget *label;

	int i;
	int a; 
	int b; 
	int res;
	int boxh;
        gchar token1[]=",";
	gchar descript[500];
/* Add memory here */
	/* where needs to match the size of programs[i].where */
	gchar where[3500];

	strcpy(where,programs[(int) data].where);

	if (strcmp(setDebug,"11")==0) {
		printf("Separator=%s	Data=%s\n",token1,where); 
	}
	res=doSplitLine(where,token1); 
	for (i=0; i< res; i++) {
		if (strcmp(setDebug,"11")==0) {
			printf("%s\n",array1[i]);
		}
	} 
	if (res == 0) {
		boxh = 50 + 120;
	} else if (res == 1) {
		boxh = 50 + 120;
	} else {
		boxh = 30*res + 130;
	}
	window2= gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (window2), "Download & Install");
	gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
	gtk_widget_set_size_request (window2, 550, boxh);

	label = gtk_label_new ("Below are the packages that may need to be installed.\nUnless you know better, you should install all of them.\nIf you do not want to install a package simply unselect it before clicking INSTALL");
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
	gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), scrolled_window, 
			TRUE, TRUE, 0);
	gtk_widget_show (scrolled_window);
    
    /* create a table of res by 1 squares. */
	if (res == 0) {
		table = gtk_table_new (1, 1, FALSE);
	} else {
		for (i=0; i< res; i++) {
			strcpy(descript,array1[i]);
			deblank(descript);
			strcpy(ftparray[i].where, descript);
			if (strcmp(setDebug,"11")==0) {
				printf("ftparray[%i] = %s\n",i,ftparray[i].where);
			}
		}	
		table = gtk_table_new (res, 1, FALSE);
	}
    
    /* set the spacing to 10 on x and 10 on y */
	gtk_table_set_row_spacings (GTK_TABLE (table), 1);
	gtk_table_set_col_spacings (GTK_TABLE (table), 1);
   
    /* pack the table into the scrolled window */
	gtk_scrolled_window_add_with_viewport (
                   GTK_SCROLLED_WINDOW (scrolled_window), table);
	gtk_widget_show (table);
  
	if (res==0) {
		i=0;
		button = gtk_check_button_new_with_label((char *) data);
		g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (toggle_button_callback), (int *) i);

		gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,0,1);
		gtk_widget_show (button);
	} else {
		a=0;
		b=1;
		ftpsize=0;
		for (i=0; i< res; i++) {
			ftpsize++;
/*			printf("Row number %i, %s\n",a,descript); */
			button = gtk_check_button_new_with_label(ftparray[i].where);
			g_signal_connect (G_OBJECT (button), "clicked",
	                      G_CALLBACK (toggle_button_callback), (int *) i);
			gtk_toggle_button_set_active((GtkToggleButton *) button, TRUE );
			gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,a,b);
	        	gtk_widget_show (button);
			a++;b++;
		}
	}

/* Add an "install" button to the bottom of the dialog */
	button = gtk_button_new_with_label ("Install");
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
				G_CALLBACK (callback2), (gpointer) "Installing");
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window2)->action_area), button, TRUE, TRUE, 0);
	gtk_widget_show (button);

/* Add a "close" button to the bottom of the dialog */
        button = gtk_button_new_with_label ("Close");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window2);

/* this makes it so the button is the default. */

        GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window2)->action_area), button, TRUE, TRUE, 0);

/* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
        gtk_widget_grab_default (button);
        gtk_widget_show (button);

	gtk_widget_show (window2);
	
}


static void callback (GtkWidget *widget,
                        gpointer data)
{
	char dataOut[20];
	char descript[200]=" ";
	gchar xpm[25]=" ";
	gchar temppath[5]=" ";
	int i;
	int a;
	int b;
	int nrows=0;
        static GtkWidget *window1;
	GtkWidget *scrolled_window;
	GtkWidget *table;
	GtkWidget *button;
	GtkWidget *box;
	GtkWidget *label;

	strcpy(dataOut,(char *) data);

	window1 = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (window1), dataOut);
	gtk_container_set_border_width (GTK_CONTAINER (window1), 0);
	gtk_widget_set_size_request (window1, 460, 320);

	/* Create a label for the window */
        label = gtk_label_new ("Select the package\nyou would like to install or upgrade");
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

        /* Pack the image and label into the window */
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), label, FALSE, FALSE, 0);
    	gtk_widget_show (label);

    /* create a new scrolled window. */
	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    
	gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    /* The dialog window is created with a vbox packed into it. */								
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), scrolled_window, 
			TRUE, TRUE, 0);
	gtk_widget_show (scrolled_window);
   
    /* create a table of nrows by 1 squares. */
	for (i=0; i< arraySize; i++) {
		if (strcmp(programs[i].cat,dataOut)==0) {
			nrows++;
		}
	}

	table = gtk_table_new (nrows, 1, FALSE);
   
    /* set the spacing to 10 on x and 10 on y */
	gtk_table_set_row_spacings (GTK_TABLE (table), 1);
	gtk_table_set_col_spacings (GTK_TABLE (table), 1);
    
    /* pack the table into the scrolled window */
	gtk_scrolled_window_add_with_viewport (
                   GTK_SCROLLED_WINDOW (scrolled_window), table);
	gtk_widget_show (table);
   
 	a=0;
	b=1;

	for (i=1; i< arraySize; i++) {
		if (strcmp(programs[i].cat,dataOut)==0) {
			strcpy(temppath,"icons/");
			strcpy(xpm,programs[i].xpix); 
			strcpy(descript,programs[i].text);
			deblank(xpm);
			strcat(temppath,xpm);
/*			printf("%s\n",temppath);  */
		        button = gtk_button_new ();
        		g_signal_connect (G_OBJECT (button), "clicked",
        		                G_CALLBACK (callback1), (gpointer) i);
        		box = xpm_label_box ( temppath, descript);

        		gtk_widget_show (box);
        		gtk_container_add (GTK_CONTAINER (button), box);
			gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,a,b);
        		gtk_widget_show (button);
/*			printf("%s\n",programs[i].text); */
			a++; b++;
		}
	}

/* Add a "close" button to the bottom of the dialog */
        button = gtk_button_new_with_label ("Close");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);

/* this makes it so the button is the default. */

        GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window1)->action_area), button, TRUE, TRUE, 0);

/* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
        gtk_widget_grab_default (button);
        gtk_widget_show (button);

	gtk_widget_show (window1);

}

static void doUninstall (GtkWidget *widget,
			gpointer data)
{

	static GtkWidget *window2;
	GtkWidget *label;
	char labelTxt1[20000] = "\nPackage Description:\n\n";
        gchar inputFileName[50]=" ";
        gchar C;
        gchar text[5000]=" ";
        FILE *inputFilePointer;
        int inputFileSize;
	gchar command1[50] = "./listdescription.sh ";
	gchar command2[50] = "./xuninstallpackage.sh ";

	/* List package description and write it to a file  */
	if (strcmp(setDebug,"11")==0) {
		printf("Current Package = %s\n",currentPackages[(int) data].where);
	}
	strcpy(inputFileName,currentPackages[(int) data].where);
	strcat(command1,inputFileName);
	if (strcmp(setDebug,"11")==0) {
		printf("%s\n",command1);
	}
	strcat(command2,inputFileName);
	system(command1);

/* 	isPackageAlreadyThere( */
	/* Open up the package description */
	strcpy(inputFileName,"package_description.txt");
        inputFilePointer = fopen(inputFileName,"r");
        inputFileSize = 0;
        while (fscanf(inputFilePointer,"%c",&C) != -1) text[inputFileSize++] = C;
	fclose(inputFilePointer);

	/* Copy the package description to the label text */
	strcat(labelTxt1,text);

	window2= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
	gtk_window_set_title (GTK_WINDOW (window2), "Do You Wish to Uninstall This Package?");
	gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
	gtk_widget_set_size_request (window2, 500, 275);

	label = gtk_label_new (labelTxt1);
/*	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER ); */

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window2), "Yes", 1);
	gtk_dialog_add_button (GTK_DIALOG (window2), "No", 2);

	gint result = gtk_dialog_run (GTK_DIALOG (window2) );
	if (strcmp(setDebug,"11")==0) {
		printf("Result = %i\n",result);
	}
	if (result == 1) {
		system(command2);
		system("rm package_description.txt");
	}
		
	gtk_widget_destroy (window2);

		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "Alert!");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 80);
	
		label = gtk_label_new ("\n\nPackage Removal Finished\n");
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);

		gtk_dialog_add_button (GTK_DIALOG (window2), "OK", 1);
		gtk_dialog_run ( GTK_DIALOG (window2) );
		gtk_widget_destroy (window2);


}


static void uninstall (GtkWidget *widget,
                        gpointer data)
{
	char dataOut[20];
	int i;
	int a;
	int b;
        static GtkWidget *window1;
	GtkWidget *scrolled_window;
	GtkWidget *table;
	GtkWidget *button;
	GtkWidget *box;
	GtkWidget *label;

	strcpy(dataOut,(char *) data);

	window1 = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (window1), dataOut);
	gtk_container_set_border_width (GTK_CONTAINER (window1), 0);
	gtk_widget_set_size_request (window1, 460, 320);

	/* Create a label for the window */
        label = gtk_label_new ("Select the package\nyou would like to uninstall");
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

        /* Pack the image and label into the window */
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), label, FALSE, FALSE, 0);
    	gtk_widget_show (label);

    /* create a new scrolled window. */
	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    
	gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    /* The dialog window is created with a vbox packed into it. */								
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), scrolled_window, 
			TRUE, TRUE, 0);
	gtk_widget_show (scrolled_window);
   
    /* create a table of nPackages by 1 squares. */
	table = gtk_table_new (nPackages, 1, FALSE);
   
    /* set the spacing to 10 on x and 10 on y */
	gtk_table_set_row_spacings (GTK_TABLE (table), 1);
	gtk_table_set_col_spacings (GTK_TABLE (table), 1);
    
    /* pack the table into the scrolled window */
	gtk_scrolled_window_add_with_viewport (
                   GTK_SCROLLED_WINDOW (scrolled_window), table);
	gtk_widget_show (table);
   
 	a=0;
	b=1;

	for (i=0; i< nPackages; i++) {
		button = gtk_button_new ();
		g_signal_connect (G_OBJECT (button), "clicked",
					G_CALLBACK (doUninstall), (gpointer) i);
		box = xpm_label_box("icons/default.xpm",currentPackages[i].where);
		gtk_widget_show (box);
		gtk_container_add (GTK_CONTAINER (button), box);
		gtk_table_attach_defaults (GTK_TABLE(table), button, 0,1,a,b);
		gtk_widget_show (button);
		a++; b++;
	}

/* Add a "close" button to the bottom of the dialog */
        button = gtk_button_new_with_label ("Close");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);

/* this makes it so the button is the default. */

        GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window1)->action_area), button, TRUE, TRUE, 0);

/* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
        gtk_widget_grab_default (button);
        gtk_widget_show (button);

	gtk_widget_show (window1);

}

static void doFTPinstall (GtkWidget *widget,
			gpointer data)
{

	static GtkWidget *window2;
	GtkWidget *label;
	char labelTxt1[200] = " ";
	char rfname[100] = "removed_packages.txt";
	gchar text[100] = " ";
	gchar *ptext[100];
        gchar inputFileName[50]=" ";
	gchar command1[200] = "echo ";
	gchar command2[50] = " > ftp_package.txt";
	gchar command3[200] = "xterm -geometry 70x10 -e wget --passive-ftp --directory-prefix=packages ";
	gchar *oldpackage[50];
	int okInstall = 2;
	int which = -1;

	*ptext = text;
	strcpy(labelTxt1,ftpPackages[(int) data].where);
	/* List package description and write it to a file  */
	if (strcmp(setDebug,"11")==0) {
		printf("Current Package = %s\n",currentPackages[(int) data].where);
	}
	strcpy(inputFileName,ftpPackages[(int) data].where);
	strcat(command1,inputFileName);
	strcat(command1,command2);
	system(command1);
	if (strcmp(setDebug,"11")==0) {
		printf("%s\n",command1);
	}

	isPackageThereAlready(inputFileName); 	/* Check to see if package is already installed */
	okInstall = packageOK.response;
	which = packageOK.which;
	if (which > -1 ) {
		*oldpackage = currentPackages[which].where;
	}
	
	if (okInstall == 1) {		/* Replace the package to be installed */
		strcat(command3,ftpsite);
		strcat(command3,"/");
		strcat(command3,inputFileName);

		*ptext = *oldpackage;

		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "OK to Install?");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 175);

		label = gtk_label_new (labelTxt1);
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER ); 
	
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);
	
		gtk_dialog_add_button (GTK_DIALOG (window2), "Yes", 1);
		gtk_dialog_add_button (GTK_DIALOG (window2), "No", 2);

		gint result = gtk_dialog_run (GTK_DIALOG (window2) );
		if (strcmp(setDebug,"11")==0) {
			printf("Result = %i\n",result);
		}
		if (result == 1) {	/* If user confirms replace, then get package */
			writeFile(rfname,*ptext);
			system(command3);
			system("./xinstallpackage.sh");
			if (strcmp(cleanUp,"1")==0) {
				system("./cleanup1.sh");
			}
		}

	} else if (okInstall == 2) { 	/* Just install the darn thing */

		strcat(command3,ftpsite);
		strcat(command3,"/");
		strcat(command3,inputFileName);

		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "OK to Install?");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 175);

		label = gtk_label_new (labelTxt1);
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER ); 
	
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);
	
		gtk_dialog_add_button (GTK_DIALOG (window2), "Yes", 1);
		gtk_dialog_add_button (GTK_DIALOG (window2), "No", 2);

		gint result = gtk_dialog_run (GTK_DIALOG (window2) );
		if (strcmp(setDebug,"11")==0) {
			printf("Result = %i\n",result);
		}
		if (result == 1) {	/* If user confirms install, then get package */
			system(command3);
			system("./xinstallpackage.sh");
			if (strcmp(cleanUp,"1")==0) {
				system("./cleanup1.sh");
			}
		}
	} else {	/* Don't install it and abort! Notify user! */
		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "Aborting Install");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 100);

		label = gtk_label_new (labelTxt1);
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER ); 
	
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);
	
		gtk_dialog_add_button (GTK_DIALOG (window2), "OK", 1);

		gtk_dialog_run (GTK_DIALOG (window2) );

	}
		
	gtk_widget_destroy (window2);

	if (okInstall < 3 ) {
		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "Alert!");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 80);
	
		label = gtk_label_new ("\n\nInstallation Finished\n");
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);

		gtk_dialog_add_button (GTK_DIALOG (window2), "OK", 1);
		gtk_dialog_run ( GTK_DIALOG (window2) );
		gtk_widget_destroy (window2);


	}
 
}

static void ftpinstall (GtkWidget *widget,
                        gpointer data)
{
	int i;
	int a;
	int b;
        static GtkWidget *window1;
	GtkWidget *scrolled_window;
	GtkWidget *table;
	GtkWidget *button;
	GtkWidget *box;
	GtkWidget *label;
/*	strcpy(dataOut,(char *) data); */

	window1 = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (window1), "Install Package");
	gtk_container_set_border_width (GTK_CONTAINER (window1), 0);
	gtk_widget_set_size_request (window1, 460, 320);

	/* Create a label for the window */
        label = gtk_label_new ("Select the package\nyou would like to install.\n\nNo dependencies will be checked.");
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

        /* Pack the image and label into the window */
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), label, FALSE, FALSE, 0);
    	gtk_widget_show (label);

    /* create a new scrolled window. */
	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    
	gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    /* The dialog window is created with a vbox packed into it. */								
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), scrolled_window, 
			TRUE, TRUE, 0);
	gtk_widget_show (scrolled_window);
   
    /* create a table of nPackages by 1 squares. */
	table = gtk_table_new (allPackages, 1, FALSE);
   
    /* set the spacing to 10 on x and 10 on y */
	gtk_table_set_row_spacings (GTK_TABLE (table), 1);
	gtk_table_set_col_spacings (GTK_TABLE (table), 1);
    
    /* pack the table into the scrolled window */
	gtk_scrolled_window_add_with_viewport (
                   GTK_SCROLLED_WINDOW (scrolled_window), table);
	gtk_widget_show (table);
   
 	a=0;
	b=1;

	for (i=0; i< allPackages; i++) {
		button = gtk_button_new ();
		g_signal_connect (G_OBJECT (button), "clicked",
					G_CALLBACK (doFTPinstall), (gpointer) i);
		box = xpm_label_box("icons/default.xpm",ftpPackages[i].where);
		gtk_widget_show (box);
		gtk_container_add (GTK_CONTAINER (button), box);
		gtk_table_attach_defaults (GTK_TABLE(table), button, 0,1,a,b);
		gtk_widget_show (button);
		a++; b++;
	}

/* Add a "close" button to the bottom of the dialog */
        button = gtk_button_new_with_label ("Close");
        gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);

/* this makes it so the button is the default. */

        GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window1)->action_area), button, TRUE, TRUE, 0);

/* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
        gtk_widget_grab_default (button);
        gtk_widget_show (button);

	gtk_widget_show (window1);

}


static void browseselect ( GtkWidget *widget,
			GtkFileSelection *fs )
{
	gchar package[100];
	gchar *ppackage[100];
	gchar packname[50];
	gchar *ppackname[50];
	gchar delim[1] = "/";
	static GtkWidget *window2;
	GtkWidget *label;
/*	char labelTxt1[200] = " "; */
	char rfname[100] = "removed_packages.txt";
	gchar text[100] = " ";
	gchar *ptext[100];
        gchar inputFileName[50]=" ";
	gchar command1[200] = "echo ";
	gchar command2[50] = " > ftp_package.txt";
	gchar *oldpackage[50];
	int okInstall = 2;
	int which = -1;
	int res = 0;

	*ptext = text; 		/* Packages to be uninstalled in txt form */
	*ppackname = packname;	/* Actual name of package */
	strcpy(package, gtk_file_selection_get_filename(GTK_FILE_SELECTION (fs)));
	/* List package description and write it to a file  */
	strcat(command1,package);
	strcat(command1,command2);
	system(command1);
	if (strcmp(setDebug,"11")==0) {
		printf("%s\n",command1);
	}

	*ppackage = package;	/* This is the name + the path */
	res = doSplitLine(*ppackage,delim);
	*ppackname = array1[res-1];
	strcpy(inputFileName,*ppackname);

	isPackageThereAlready(inputFileName);
	okInstall = packageOK.response;
	which = packageOK.which;
	if (which > -1 ) {
		*oldpackage = currentPackages[which].where;
	}
	
	if (okInstall == 1) {		/* Replace the package to be installed */
		*ptext = *oldpackage;

		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "OK to Install?");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 175);

		label = gtk_label_new (*ppackname);
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER ); 
	
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);
	
		gtk_dialog_add_button (GTK_DIALOG (window2), "Yes", 1);
		gtk_dialog_add_button (GTK_DIALOG (window2), "No", 2);

		gint result = gtk_dialog_run (GTK_DIALOG (window2) );
		if (strcmp(setDebug,"11")==0) {
			printf("Result = %i\n",result);
		}
		if (result == 1) {
			writeFile(rfname,*ptext);
			system("./xinstallhdpackage.sh");
		}

	} else if (okInstall == 2) { 	/* Just install the darn thing */

		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "OK to Install?");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 175);

		label = gtk_label_new (*ppackname);
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER ); 
	
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);
	
		gtk_dialog_add_button (GTK_DIALOG (window2), "Yes", 1);
		gtk_dialog_add_button (GTK_DIALOG (window2), "No", 2);

		gint result = gtk_dialog_run (GTK_DIALOG (window2) );
		if (strcmp(setDebug,"11")==0) {
			printf("Result = %i\n",result);
		}
		if (result == 1) {
			system("./xinstallhdpackage.sh");
		}
	} else {	/* Don't install it and abort! */
		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "Aborting Install");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 100);

		label = gtk_label_new (*ppackname);
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER ); 
	
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);
	
		gtk_dialog_add_button (GTK_DIALOG (window2), "OK", 1);

		gtk_dialog_run (GTK_DIALOG (window2) );

	}
		
	gtk_widget_destroy (window2);
	if (okInstall < 3 ) {
		window2= gtk_dialog_new ();
		gtk_window_set_modal(GTK_WINDOW(window2), TRUE);
		gtk_window_set_title (GTK_WINDOW (window2), "Alert!");
		gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
		gtk_widget_set_size_request (window2, 250, 120);
	
		label = gtk_label_new ("\nInstallation Finished\n");
		gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
		gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
		gtk_widget_show (label);

		gtk_dialog_add_button (GTK_DIALOG (window2), "OK", 1);
		gtk_dialog_run ( GTK_DIALOG (window2) );
		gtk_widget_destroy (window2);


	}

}

static void browseinstall( GtkWidget *widget,
			gpointer data )
{
    GtkWidget *filew;
    /* Create a new file selection widget */
    filew = gtk_file_selection_new ("File selection");
    
    g_signal_connect (G_OBJECT (filew), "destroy",
	              G_CALLBACK (gtk_widget_destroy), NULL);
    /* Connect the ok_button to file_ok_sel function */
    g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		      "clicked", G_CALLBACK (browseselect), (gpointer) filew);
    g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
	                      "clicked", G_CALLBACK (gtk_widget_destroy),
			      G_OBJECT (filew));
    
    /* Connect the cancel_button to destroy the widget */
    g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button),
	                      "clicked", G_CALLBACK (gtk_widget_destroy),
			      G_OBJECT (filew));
    
    gtk_widget_show (filew);
    if (strcmp(setDebug,"11")==0) {
	printf("This will allow the user to install a package by browsing their hd\n");
    }
}

static void destroy( GtkWidget *widget,
                     gpointer   data )
{
    gtk_main_quit ();
}

static void doInstall (GtkWidget *widget,
			gpointer data)
{
    static GtkWidget *window1;
    GtkWidget *scrolled_window;
    GtkWidget *table;
    GtkWidget *button;
    GtkWidget *box;
    GtkWidget *label;

	/* Now start building widgets */

	window1 = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (window1), "Program Groups");
	gtk_container_set_border_width (GTK_CONTAINER (window1), 0);
	gtk_widget_set_size_request (window1, 260, 320);

	/* Create a label for the window */
        label = gtk_label_new ("Select the type of package\nyou would like to install or upgrade");
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

        /* Pack the image and label into the window */
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), label, FALSE, FALSE, 0);
    	gtk_widget_show (label);

    /* create a new scrolled window. */
	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    
	gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
    
    /* the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
     * GTK_POLICY_AUTOMATIC will automatically decide whether you need
     * scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
     * there.  The first one is the horizontal scrollbar, the second, 
     * the vertical. */
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    /* The dialog window is created with a vbox packed into it. */								
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), scrolled_window, 
			TRUE, TRUE, 0);
	gtk_widget_show (scrolled_window);
    
    /* create a table of 12 by 1 squares. */
	table = gtk_table_new (12, 1, FALSE);
    
    /* set the spacing to 1 on x and 1 on y */
	gtk_table_set_row_spacings (GTK_TABLE (table), 1);
	gtk_table_set_col_spacings (GTK_TABLE (table), 1);
    
    /* pack the table into the scrolled window */
	gtk_scrolled_window_add_with_viewport (
                   GTK_SCROLLED_WINDOW (scrolled_window), table);
	gtk_widget_show (table);
    
/* Add Office button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (callback), (gpointer) "office");
        box = xpm_label_box ("icons/office.xpm", "Office and Productivity");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,0,1);
        gtk_widget_show (button);

/* Add Finance button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (callback), (gpointer) "finance");
        box = xpm_label_box ("icons/financial.xpm", "Financial Software");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,1,2);
        gtk_widget_show (button);

/* Add Media button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (callback), (gpointer) "media");
        box = xpm_label_box ("icons/audio.xpm", "Music and Multimedia");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,2,3);
        gtk_widget_show (button);

/* Add Internet Button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (callback), (gpointer) "internet");
        box = xpm_label_box ("icons/internet.xpm", "Internet and Connectivity");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,3,4);
        gtk_widget_show (button);
	
/* Add Graphics Button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (callback), (gpointer) "graphics");
        box = xpm_label_box ("icons/graphics.xpm", "Graphics");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,4,5);
        gtk_widget_show (button);
	
/* Add Games Button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (callback), (gpointer) "games");
        box = xpm_label_box ("icons/games.xpm", "Games");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,5,6);
        gtk_widget_show (button);

/* Add Science Button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (callback), (gpointer) "science");
        box = xpm_label_box ("icons/science.xpm", "Science");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,6,7);
        gtk_widget_show (button);
	

/* Add Window Manager Button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (callback), (gpointer) "windows");
        box = xpm_label_box ("icons/wm.xpm", "Window Managers");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,7,8);
        gtk_widget_show (button);

/* Add System Administration Button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (callback), (gpointer) "system");
        box = xpm_label_box ("icons/systemadmin.xpm", "System Administration");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,8,9);
        gtk_widget_show (button);

/* Add Other Button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (callback), (gpointer) "other");
        box = xpm_label_box ("icons/default.xpm", "Other Applications");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,9,10);
        gtk_widget_show (button);
	
/* Add Browse All Button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (ftpinstall), (gpointer) "other");
        box = xpm_label_box ("icons/pkgselect.xpm", "Package List");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,10,11);
        gtk_widget_show (button);

/* Add Browse Computer */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (browseinstall), (gpointer) "other");
        box = xpm_label_box ("icons/harddrive.xpm", "Browse Computer");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,11,12);
        gtk_widget_show (button);


	
/* Add a "close" button to the bottom of the dialog */
	button = gtk_button_new_with_label ("Close");
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);

/* this makes it so the button is the default. */
    
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window1)->action_area), button, TRUE, TRUE, 0);
    
/* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
	gtk_widget_grab_default (button);
	gtk_widget_show (button);
    
	gtk_widget_show (window1);

}

static void saveprefs() 
{
	gchar txtOut[1000]=" ";

	strcpy(txtOut,ftpsite);	/* Default location of files to be downloaded from pkglist.txt */
	strcat(txtOut,"\n");
	strcat(txtOut,manifestsite);
	strcat(txtOut,"\n");
	strcat(txtOut,setDebug);	/* Integer 1= setDebug on, 0= off */
	writeFile("config.txt",txtOut);
	
}

static void updateManifest (GtkWidget *widget,
			GtkWidget *data) 
{
	static GtkWidget *window3;
	const gchar *site;
	GtkWidget *label;
	char labelTxt1[1000] = "Default location of manifest server is now:\n";

	site = gtk_entry_get_text (GTK_ENTRY (data));
	strcat(labelTxt1,site);
	strcat(labelTxt1,"\n\n");
	strcat(labelTxt1,"Press OK to accept or Cancel to reject.");

	window3= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window3), TRUE);
	gtk_window_set_title (GTK_WINDOW (window3), "Manifest Server");
	gtk_container_set_border_width (GTK_CONTAINER (window3), 0);
	gtk_widget_set_size_request (window3, 600, 150);

	label = gtk_label_new (labelTxt1);
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window3)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window3), "OK", 1);
	gtk_dialog_add_button (GTK_DIALOG (window3), "Cancel", 2);

	gint result = gtk_dialog_run (GTK_DIALOG (window3) );
	if (strcmp(setDebug,"11")==0) {
		printf("Result = %i\n",result);
	}

	gtk_widget_destroy (window3);
	if (result == 1) {
	  strcpy(manifestsite,site);
	  if (strcmp(setDebug,"11")==0) {
		  printf ("Entry contents: %s\n", manifestsite);
	  }
	  saveprefs();
	}
}

static void updatePkgList (GtkWidget *widget,
			GtkWidget *data) 
{
	static GtkWidget *window3;
	const gchar *site;
	GtkWidget *label;
	char labelTxt1[1000] = "Default location of manifest server is now:\n";

	site = gtk_entry_get_text (GTK_ENTRY (data));
	strcat(labelTxt1,site);
	strcat(labelTxt1,"\n\n");
	strcat(labelTxt1,"Press OK to accept or Cancel to reject.");

	window3= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window3), TRUE);
	gtk_window_set_title (GTK_WINDOW (window3), "Package List Server");
	gtk_container_set_border_width (GTK_CONTAINER (window3), 0);
	gtk_widget_set_size_request (window3, 600, 150);

	label = gtk_label_new (labelTxt1);
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window3)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window3), "OK", 1);
	gtk_dialog_add_button (GTK_DIALOG (window3), "Cancel", 2);

	gint result = gtk_dialog_run (GTK_DIALOG (window3) );
	if (strcmp(setDebug,"11")==0) {
		printf("Result = %i\n",result);
	}

	gtk_widget_destroy (window3);
	if (result == 1) {
	  strcpy(ftpsite,site);
	  if (strcmp(setDebug,"11")==0) {
		  printf ("Entry contents: %s\n", manifestsite);
	  }
	  saveprefs();
	}
}


static void setmanifest (GtkWidget *widget,
			gpointer data)
{
    GtkWidget *entry1;
    GtkWidget *entry2;
    GtkWidget *label;
    GtkWidget *window2;
    GtkWidget *box2;
    GtkWidget *button;
    /*GtkWidget *separator; */
  
    window2 = gtk_dialog_new ();
  
    g_signal_connect (G_OBJECT (window2), "delete_event",
			GTK_SIGNAL_FUNC (gtk_widget_destroy), window2);

    gtk_window_set_title (GTK_WINDOW (window2), "Set Manifest Server");
    gtk_container_set_border_width (GTK_CONTAINER (window2), 0);
    gtk_widget_set_size_request (window2, 500, 220);

/* Create a label for the window */
    label = gtk_label_new ("\nChoose the servers for the package manifest\nand the package list.\nPress RETURN in a text field to update the values.");
    gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

/* Pack the image and label into the window */
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window2)->vbox), label, FALSE, FALSE, 0);
   		gtk_widget_show (label);

    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_start(GTK_BOX (GTK_DIALOG(window2)->vbox),box2, TRUE, TRUE, 0);
    gtk_widget_show (box2);


    label = gtk_label_new ("Internet address of package_manifest.txt");
    gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 0);
    gtk_widget_show (label);

    entry1 = gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry1), 100);
    g_signal_connect (G_OBJECT (entry1), "activate",
		      G_CALLBACK (updateManifest),
		      (gpointer) entry1);
    gtk_entry_set_text (GTK_ENTRY (entry1), manifestsite);
    gtk_box_pack_start (GTK_BOX (box2), entry1, FALSE, FALSE, 0);
    gtk_widget_show (entry1);

    label = gtk_label_new ("Internet address of pkglist.txt");
    gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );
    gtk_box_pack_start (GTK_BOX (box2), label, FALSE, FALSE, 0);
    gtk_widget_show (label);

    entry2= gtk_entry_new ();
    gtk_entry_set_max_length (GTK_ENTRY (entry2), 100);
    g_signal_connect (G_OBJECT (entry2), "activate",
		      G_CALLBACK (updatePkgList),
		      (gpointer) entry2);
    gtk_entry_set_text (GTK_ENTRY (entry2), ftpsite);
    gtk_box_pack_start (GTK_BOX (box2), entry2, FALSE, FALSE, 0);
    gtk_widget_show (entry2);


/* Add a "close" button to the bottom of the dialog */
	button = gtk_button_new_with_label ("Close");
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (gtk_widget_destroy), window2);

/* this makes it so the button is the default. */
    
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window2)->action_area), button, TRUE, TRUE, 0);
    
/* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
	gtk_widget_grab_default (button);
	gtk_widget_show (button);
    
	gtk_widget_show (window2);
}

static void getmanifest (GtkWidget *widget,
			gpointer data)
{
	static GtkWidget *window3;
	GtkWidget *label;
	char labelTxt1[1000] = "\nAre you sure you want to update your manifests?\nThis requires an internet connection.";
	char command1[1000] = " ";

	window3= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window3), TRUE);
	gtk_window_set_title (GTK_WINDOW (window3), "Update Manifests?");
	gtk_container_set_border_width (GTK_CONTAINER (window3), 0);
	gtk_widget_set_size_request (window3, 300, 120);

	label = gtk_label_new (labelTxt1);
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window3)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window3), "OK", 1);
	gtk_dialog_add_button (GTK_DIALOG (window3), "Cancel", 2);

	gint result = gtk_dialog_run (GTK_DIALOG (window3) );
	if (strcmp(setDebug,"11")==0) {
		printf("Result = %i\n",result);
	}

	gtk_widget_destroy (window3);
	if (result == 1) {
		strcpy(command1,"./xgetmanifests.sh");
		strcat(command1," ");
		strcat(command1,manifestsite);
		strcat(command1," ");
		strcat(command1,ftpsite);
		if (strcmp(setDebug,"11")==0) {
			printf("The command is: %s\n",command1);
		}
		system(command1);
		getManifest();
	}

}

static void localManifestSelect (GtkWidget *widget,
			GtkFileSelection *fs)
{
	gchar fname[100];
	gchar command1[100] = "cp ";
	gchar labelTxt1[100] = "\nThis may change which packages you can install.";
	GtkWidget *label;
	static GtkWidget *window3;

	strcpy(fname,gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
	strcat(command1,fname);
	strcat(command1," ./package_manifest.txt");

	window3= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window3), TRUE);
	gtk_window_set_title (GTK_WINDOW (window3), "Change Manifest?");
	gtk_container_set_border_width (GTK_CONTAINER (window3), 0);
	gtk_widget_set_size_request (window3, 300, 120);

	label = gtk_label_new (labelTxt1);
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window3)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window3), "OK", 1);
	gtk_dialog_add_button (GTK_DIALOG (window3), "Cancel", 2);

	gint result = gtk_dialog_run (GTK_DIALOG (window3) );
	if (strcmp(setDebug,"11")==0) {
		printf("Result = %i\n",result);
	}

	gtk_widget_destroy (window3);
	if (result == 1) {
		system(command1);
		getManifest();
	}


}

static void chooseLocalManifest (GtkWidget *widget,
			gpointer data)
{
    GtkWidget *filew;
    /* Create a new file selection widget */
    filew = gtk_file_selection_new ("File selection");
    
    g_signal_connect (G_OBJECT (filew), "destroy",
	              G_CALLBACK (gtk_widget_destroy), NULL);
    /* Connect the ok_button to file_ok_sel function */
    g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		      "clicked", G_CALLBACK (localManifestSelect), (gpointer) filew);
    g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
	                      "clicked", G_CALLBACK (gtk_widget_destroy),
			      G_OBJECT (filew));
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew), "manifests/");
    
    /* Connect the cancel_button to destroy the widget */
    g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button),
	                      "clicked", G_CALLBACK (gtk_widget_destroy),
			      G_OBJECT (filew));
    
    gtk_widget_show (filew);

}

static void localPkglistSelect (GtkWidget *widget,
			GtkFileSelection *fs)
{
	gchar fname[100];
	gchar command1[100] = "cp ";
	gchar labelTxt1[100] = "\nThis may change which packages you can install.";
	GtkWidget *label;
	static GtkWidget *window3;

	strcpy(fname,gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs)));
	strcat(command1,fname);
	strcat(command1," ./pkglist.txt");

	window3= gtk_dialog_new ();
	gtk_window_set_modal(GTK_WINDOW(window3), TRUE);
	gtk_window_set_title (GTK_WINDOW (window3), "Change Package List?");
	gtk_container_set_border_width (GTK_CONTAINER (window3), 0);
	gtk_widget_set_size_request (window3, 300, 120);

	label = gtk_label_new (labelTxt1);
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window3)->vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	gtk_dialog_add_button (GTK_DIALOG (window3), "OK", 1);
	gtk_dialog_add_button (GTK_DIALOG (window3), "Cancel", 2);

	gint result = gtk_dialog_run (GTK_DIALOG (window3) );
	if (strcmp(setDebug,"11")==0) {
		printf("Result = %i\n",result);
	}

	gtk_widget_destroy (window3);
	if (result == 1) {
		system(command1);
		getManifest();
	}


}

static void chooseLocalPkglist (GtkWidget *widget,
			gpointer data)
{
    GtkWidget *filew;
    /* Create a new file selection widget */
    filew = gtk_file_selection_new ("File selection");
    
    g_signal_connect (G_OBJECT (filew), "destroy",
	              G_CALLBACK (gtk_widget_destroy), NULL);
    /* Connect the ok_button to file_ok_sel function */
    g_signal_connect (G_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		      "clicked", G_CALLBACK (localPkglistSelect), (gpointer) filew);
    g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
	                      "clicked", G_CALLBACK (gtk_widget_destroy),
			      G_OBJECT (filew));
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(filew), "lists/");
    
    /* Connect the cancel_button to destroy the widget */
    g_signal_connect_swapped (G_OBJECT (GTK_FILE_SELECTION (filew)->cancel_button),
	                      "clicked", G_CALLBACK (gtk_widget_destroy),
			      G_OBJECT (filew));
    
    gtk_widget_show (filew);


}


static void setup (GtkWidget *widget,
			gpointer data)
{
    static GtkWidget *window1;
    GtkWidget *scrolled_window;
    GtkWidget *table;
    GtkWidget *button;
    GtkWidget *box;
    GtkWidget *label;

	/* Now start building widgets */

	window1 = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (window1), "Configure");
	gtk_container_set_border_width (GTK_CONTAINER (window1), 0);
	gtk_widget_set_size_request (window1, 300, 320);

	/* Create a label for the window */
        label = gtk_label_new ("\nChoose the settings you wish to modify below.\n");
	gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

        /* Pack the image and label into the window */
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), label, FALSE, FALSE, 0);
    	gtk_widget_show (label);

    /* create a new scrolled window. */
	scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    
	gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
    
    /* the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
     * GTK_POLICY_AUTOMATIC will automatically decide whether you need
     * scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
     * there.  The first one is the horizontal scrollbar, the second, 
     * the vertical. */
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    /* The dialog window is created with a vbox packed into it. */								
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window1)->vbox), scrolled_window, 
			TRUE, TRUE, 0);
	gtk_widget_show (scrolled_window);
    
    /* create a table of 4 by 1 squares. */
	table = gtk_table_new (4, 1, FALSE);
    
    /* set the spacing to 1 on x and 1 on y */
	gtk_table_set_row_spacings (GTK_TABLE (table), 1);
	gtk_table_set_col_spacings (GTK_TABLE (table), 1);
    
    /* pack the table into the scrolled window */
	gtk_scrolled_window_add_with_viewport (
                   GTK_SCROLLED_WINDOW (scrolled_window), table);
	gtk_widget_show (table);
    
/* Add Set Manifest button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (setmanifest), (gpointer) "Set Manifest");
        box = xpm_label_box ("icons/server.xpm", "Set Package Manifest Server");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,0,1);
        gtk_widget_show (button);

/* Add Get Manifest button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (getmanifest), (gpointer) "Get Manifest");
        box = xpm_label_box ("icons/internet.xpm", "Get Package Manifest from Server");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,1,2);
        gtk_widget_show (button);

/* Add Choose Local Manifest button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (chooseLocalManifest), (gpointer) "Local Manifest");
        box = xpm_label_box ("icons/pkgselect.xpm", "Choose Local Manifest");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,2,3);
        gtk_widget_show (button);

/* Add Set Installation Prefs Button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (chooseLocalPkglist), (gpointer) "Local Package List");
        box = xpm_label_box ("icons/pkgselect.xpm", "Choose Local Package List");

        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,3,4);
        gtk_widget_show (button);
	
/* Add a "close" button to the bottom of the dialog */
	button = gtk_button_new_with_label ("Close");
	gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC (gtk_widget_destroy), window1);

/* this makes it so the button is the default. */
    
	GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window1)->action_area), button, TRUE, TRUE, 0);
    
/* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
	gtk_widget_grab_default (button);
	gtk_widget_show (button);
    
	gtk_widget_show (window1);

}


int main( int   argc,
          char *argv[] )
{
    static GtkWidget *window;
    GtkWidget *scrolled_window;
    GtkWidget *table;
    GtkWidget *button;
    GtkWidget *box;
    GtkWidget *label;
    
    gtk_init (&argc, &argv);
    initArr();
    getManifest();
    
    /* Create a new dialog window for the scrolled window to be
     * packed into.  */
    window = gtk_dialog_new ();
    g_signal_connect (G_OBJECT (window), "destroy",
		      G_CALLBACK (destroy), NULL);
    gtk_window_set_title (GTK_WINDOW (window), "VL Easy Installer");
    gtk_container_set_border_width (GTK_CONTAINER (window), 0);
    gtk_widget_set_size_request (window, 260, 260);

/* Create a label for the window */
    label = gtk_label_new ("Vector Linux EZInstaller\nClick on a Button to Continue");
    gtk_label_set_justify ( GTK_LABEL(label), GTK_JUSTIFY_CENTER );

/* Pack the image and label into the window */
        gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window)->vbox), label, FALSE, FALSE, 0);
    	gtk_widget_show (label);

/* create a new scrolled window. */
    scrolled_window = gtk_scrolled_window_new (NULL, NULL);
    
    gtk_container_set_border_width (GTK_CONTAINER (scrolled_window), 10);
    
    /* the policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
     * GTK_POLICY_AUTOMATIC will automatically decide whether you need
     * scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
     * there.  The first one is the horizontal scrollbar, the second, 
     * the vertical. */
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
    /* The dialog window is created with a vbox packed into it. */								
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG(window)->vbox), scrolled_window, 
			TRUE, TRUE, 0);
    gtk_widget_show (scrolled_window);
    
    /* create a table of 3 by 1 squares. */
    table = gtk_table_new (4, 1, FALSE);
    
    /* set the spacing to 1 on x and 1 on y */
    gtk_table_set_row_spacings (GTK_TABLE (table), 1);
    gtk_table_set_col_spacings (GTK_TABLE (table), 1);
    
    /* pack the table into the scrolled window */
    gtk_scrolled_window_add_with_viewport (
                   GTK_SCROLLED_WINDOW (scrolled_window), table);
    gtk_widget_show (table);
    
/* Add Install button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (doInstall), (gpointer) "Install a Package");
        box = xpm_label_box ("icons/default.xpm", "Install a Package");

        /* Pack and show the widgets */
        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,0,1);
        gtk_widget_show (button);

/* Add Uninstall Button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (uninstall), (gpointer) "Uninstall a Package");
        box = xpm_label_box ("icons/trash.xpm", "Uninstall a Package");

        /* Pack and show the widgets */
        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,1,2);
        gtk_widget_show (button);
	
/* Add Configuration Button */
        button = gtk_button_new ();
        g_signal_connect (G_OBJECT (button), "clicked",
                        G_CALLBACK (setup), (gpointer) "Configure Easy Installer");
        box = xpm_label_box ("icons/setup.xpm", "Configure Easy Installer");

        /* Pack and show the widgets */
        gtk_widget_show (box);
        gtk_container_add (GTK_CONTAINER (button), box);
	gtk_table_attach_defaults( GTK_TABLE(table), button, 0,1,3,4);
        gtk_widget_show (button);

    /* Add a "close" button to the bottom of the dialog */
    button = gtk_button_new_with_label ("Close");
    g_signal_connect_swapped (G_OBJECT (button), "clicked",
			      G_CALLBACK (gtk_widget_destroy),
			      G_OBJECT (window));
    
    /* this makes it so the button is the default. */
    
    GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->action_area), button, TRUE, TRUE, 0);
    
    /* This grabs this button to be the default button. Simply hitting
     * the "Enter" key will cause this button to activate. */
    gtk_widget_grab_default (button);
    gtk_widget_show (button);
    
    gtk_widget_show (window);
    
    gtk_main();
    
    return 0;
}


